c
c-----------------------------------------------------------------------
c
      real*8 function fewald (wwax, wway, wwaz, qq, nsh, ierr)
c.......................................................................
c
c.....fewald - computes the electrostatic potential of a point charge
c     lattice at a point of coordinates (wwax,wway,wwaz) and charge qq.
c     The electric charge of selected lattice ions may be modified.
c     The considered test charge is -1.
c
c.....units of the output values of the potential: [r]**(-1), being
c     [r] the units used in the cell distances.
c
c.....original version by: a.g.piken and w.van gool
c                          scientific laboratory, ford motor co.
c                          dearnborn, michigan.
c                          j. mater. sci. 4(1968)95.
c                          ford tech. report #s168-10 (1968)
c
c.....modified by: j.manuel recio, june 1986.
c     departamento de quimica fisica, facultad de quimica.
c     universidad de oviedo, 33007-oviedo, spain.
c     earn: cmsqu6 at eovuov11.
c
c.....last release: a.m. pendas, 19-10-90
c     -the program  no longer sums a rigid number of shells.
c      the program will sum up as many shells as necessary to achieve
c      a predetermined convergency in potential (shthr)
c     -the old-fashioned function to compute erfc has been substituted
c      by numerical recipes analogues.
c     -some variable names have been changed by more clear ones.
c     -a new function dis2 has been added. it computes squared modulus
c      of the distance between two fractional lattice vectors.
c
c.......................................................................
c
c.....input parameters:
c     wwax,wway,wwaz ... coordinates of the point at which the potential
c                        will be computed.
c     qq ............... charge of the point.
c
c.....output parameters:
c     nsh,ierr ......... number of shells summed up and error code
c
c     ierr definition:
c                    success: ierr=0    (case a)
c            mlayer exceeded: ierr=-1   (case b)
c         abs (ew).lt.erfcth: ierr=-2   (case c)
c        case b .and. case c: ierr=-3   (case d)
c
c.......................................................................
      include           'implicit.inc'
      include           'consts.inc'
      include           'molec.inc'
      include           'ewald.inc'
      include           'stdio.inc'
c
      real*8            wwax, wway, wwaz, qq
      integer           nsh, ierr
c
c.....local variables:
c
      real*8            shthr, rfive, pfive, zfive, erfcth
      integer           mlayer
      parameter         (shthr=1d-7)
      parameter         (rfive=0.4999999d0, pfive=.5000001d0)
      parameter         (zfive=0.5d0, mlayer=15, erfcth=4.5d0)
c
      integer           ictr, icx, icy, icz, i, jack, ixx, iyy, izz
      integer           iabsix, istep
      real*8            wax, way, waz
      real*8            xt, yt, zt, ew, ewsh, within, vk
      real*8            dis, clusv, eve, aux, t
      real*8            xx(MCELL), yy(MCELL), zz(MCELL), qshift(MCELL)
      real*8            r(MCELL), eerfc(MCELL)
      logical           done
      real*8            dis2, distance, erfcc
c
c.....eta and omega are computed in the preewald routine.
c
c.....Initialize ierr
c
      ierr=0
c
c.....computing coordinates of unit-cell ions in the shifted frame:
c
      if (wwax .ge. 0d0) then
         wax = wwax - int(wwax)
      else
         wax = wwax - int(wwax) + 1
      endif
      if (wway .ge. 0d0) then
         way = wway - int(wway)
      else
         way = wway - int(wway) + 1
      endif
      if (wwaz .ge. 0d0) then
         waz = wwaz - int(wwaz)
      else
         waz = wwaz - int(wwaz) + 1
      endif
      ictr=0
      do 20 icx=-1,1
         do 22 icy=-1,1
            do 24 icz=-1,1
               do 26 jack=1,ncell
                  if (xcell(jack).ne.1d0 .and. ycell(jack).ne.1d0 .and.
     &                zcell(jack).ne.1d0) then
                     xt=xcell(jack)+icx-wax
                     yt=ycell(jack)+icy-way
                     zt=zcell(jack)+icz-waz
                     within=max(abs(xt),abs(yt),abs(zt))
                     if (within.le.pfive) then
                        ictr=ictr+1
                        xx(ictr)=xt
                        yy(ictr)=yt
                        zz(ictr)=zt
*mdc*if allow_fractional_occupancies
*                        qshift(ictr)=qcell(jack)*xocup(typcell(jack))
*mdc*else
                        qshift(ictr)=qneq(typcell(jack))
*mdc*endif
                     endif
                  endif
 26            continue
 24         continue
 22      continue
 20   continue
c
c.....computing evjen fractional charges in the shifted frame:
c
      do 30 i=1,ictr
         if (abs(xx(i)).ge.rfive) qshift(i)=zfive*qshift(i)
         if (abs(yy(i)).ge.rfive) qshift(i)=zfive*qshift(i)
         if (abs(zz(i)).ge.rfive) qshift(i)=zfive*qshift(i)
 30   continue
c
c.....computing ewald potential:
c
      ew=zero
c
c.....shell summation
c     there is no initial supposition of the number of shells to be
c     summed up. we simmulate a do while proposition
c     nsh is the ordinal number of the current shell.
c
      nsh=-1
c
c.....simulated do while
c-----do while (abs(ewsh) .gt. shthr)
      done = .false.
 10   if (.not.done) then
         nsh=nsh+1
         ewsh=zero
         do 50 ixx=-nsh,nsh
            iabsix=abs(ixx)
            do 52 iyy=-nsh,nsh
c
c..............we include only those z's that make up the shell.
c
               if (abs(iyy).eq.nsh .or. iabsix.eq.nsh) then
                  istep=1
               else
                  istep=nsh+nsh
               endif
               do 54 izz=-nsh,nsh,istep
                  if ( (ixx.ge.0) .and. (ixx.ne.0 .or. iyy.ge.0) .and.
     &                 (ixx.ne.0 .or. iyy.ne.0 .or. izz.gt.0) ) then
c
c....................vk is the squared modulus of reciprocal vector k.
c
                     vk=dis2(dble(ixx),dble(iyy),dble(izz),ar,br)
                     eve=zero
                     do 56 i=1,ictr
                        eve=eve+(qshift(i)/(pi*omega)) *
     &                    (cos(two*pi*(ixx*xx(i)+iyy*yy(i)+izz*zz(i)))*
     &                    exp(-(pi*eta)**two*vk)) /vk
 56                  continue
                     ewsh=ewsh+eve+eve
                  endif
                  do 58 i=1,ictr
c
c....................r will be the modulus of direct space vector.
c
                     aux=dis2(xx(i)+ixx,yy(i)+iyy,zz(i)+izz,aa,cosdir)
                     r(i)=sqrt(aux)
                     t=r(i)/eta
                     if (t .lt. erfcth) then
                        eerfc(i)=erfcc(t)
                     else
                        eerfc(i)=zero
                     endif
 58               continue
c
                  eve=zero
                  do 59 i=1,ictr
c
c....................exclude prime in summation
c
                     if (r(i).gt.1d-3) then
                        eve=eve+qshift(i)*eerfc(i)/r(i)
                     endif
 59               continue
                  ewsh=ewsh+eve
 54            continue
 52         continue
 50      continue
c
c........adding the shell potential to the total potential
c
         ew=ew+ewsh
c
         if (nsh. gt. mlayer) then
            write (stderr,700) wwax, wway, wway
            ierr = -1
            done = .true.
         else if (abs(ewsh) .lt. shthr)  then
            ierr = 0
            done = .true.
         endif
         goto 10
      endif
c-----end while
c
c.....include self-potential correction if needed
c.....signal if abs(ew) is less than erfcc precision 1d-7
c
      ew=ew-2*qq/(dsqrt(pi)*eta)
      if (abs(ew) .lt.  shthr .and. ew .ne. 0d0) then
         write (stderr,705) wwax, wway, wwaz, ew
         ierr = ierr-2
      endif
c
c.....compute the modified potential:
c
      clusv = zero
      do 70 i = 1,nmod
c
c...........dis is the modulus of direct space difference vector.
c
         dis = distance (xmod(i), ymod(i), zmod(i), wwax, wway, wwaz)
         if (dis .gt. 1e-3) 
     &      clusv = clusv - (qmod(i,1)-qmod(i,2))/dis
 70   continue
c
c.....modified positions potential in the inverse-distance units:
c
      fewald = - ew - clusv
      return
 700  format (1x, 'fewald: nsh > mlayer for ', 4f15.6)
 705  format (1x, 'fewald: ew value is less than expected precision!' /
     &  1x, 'wax..waz :', 3f15.6, 5x, 'ew :',1pd15.6)
      end
